<?php
/**
 * This file contains all the ajax calls used for Admin side of the addon.
 *
 * @since 1.0.2
 *
 * @package MonsterInsights
 * @subpackage MonsterInsights_User_Journey
 */

/**
 * MonsterInsights_User_Journey_Ajax class
 *
 * @since 1.0.2
 */
class MonsterInsights_User_Journey_Ajax extends MonsterInsights_User_Journey_Metabox {

	/**
	 * Provider Name.
	 *
	 * @since 1.0.2
	 *
	 * @var string
	 */
	private $provider;

	/**
	 * class constructor
	 *
	 * @return void
	 * @since 1.0.2
	 *
	 */
	public function __construct() {
		add_action( 'wp_ajax_monsterinsights_paginate_user_journey', array(
			$this,
			'monsterinsights_paginate_user_journey'
		) );
	}

	/**
	 * Display Metabox
	 * This is an abstract function in the base class.
	 *
	 * @param Object|Array $order_info Order Information
	 *
	 * @return void
	 * @since 1.0.2
	 *
	 */
	public function display_meta_box( $order_info ) {
	}

	/**
	 * Get current provider name.
	 *
	 * @return string
	 * @since 1.0.2
	 *
	 */
	protected function get_provider() {
		return $this->provider;
	}

	/**
	 * Ajax Result for pagination.
	 *
	 * @return void
	 * @since 1.0.2
	 *
	 */
	public function monsterinsights_paginate_user_journey() {

		if ( ! wp_verify_nonce( $_POST['nonce'], 'monsterinsights-uj-admin-ajax' ) ) {
			return;
		}

		if ( ! isset( $_POST['page'] ) ) {
			return;
		}

		if ( ! isset( $_POST['id'] ) ) {
			return;
		}

		$id       = absint( sanitize_text_field( $_POST['id'] ) );
		$page     = absint( sanitize_text_field( $_POST['page'] ) );
		$provider = sanitize_text_field( $_POST['provider'] );
		$page     -= 1;
		$offset   = $page * $this->db_limit();
		$order    = array();

		$this->provider = $provider;

		$order = $this->get_provider_order_data( $id );

		if ( empty( $order ) ) {
			$order['id']   = 0;
			$order['date'] = '';
		}

		$user_journey = monsterinsights_user_journey()->db->get_user_journey(
			$order['id'],
			array(
				'offset' => $offset,
				'number' => $this->db_limit()
			)
		);

		ob_start();
		$this->metabox_html( $user_journey, $order['id'], $order['date'] );
		wp_die();
	}

	/**
	 * Metabox Title.
	 *
	 * @return void
	 * @since 1.0.2
	 *
	 */
	protected function metabox_title() {
		if ( 'woocommerce' !== $this->get_provider() && 'lifterlms' !== $this->get_provider() ) {
			?>
			<div class="postbox-header">
				<h2><?php esc_html_e( 'User Journey by MonsterInsights', 'monsterinsights-user-journey' ); ?></h2>
			</div>
			<?php
		}
	}
}

// Initialize the class.
new MonsterInsights_User_Journey_Ajax();
