<?php
/**
 * User Journey MemberPress Processing.
 *
 * @since 1.0.2
 *
 * @package MonsterInsights
 * @subpackage MonsterInsights_User_Journey
 */

/**
 * Class to process user journey for MemberPress.
 *
 * This class extends MonsterInsights_User_Journey_Process base class for interactivity with
 * databse.
 *
 * @since 1.0.2
 */
class MonsterInsights_User_Journey_Process_MemberPress extends MonsterInsights_User_Journey_Process {

	/**
	 * MemberPress Payment Statues to Process
	 *
	 * @var array
	 */
	public $hooks_to_process = array( 'mepr-txn-status-complete', 'mepr-txn-status-confirmed' );

	/**
	 * Initialize.
	 *
	 * @since 1.0.2
	 */
	public function __construct() {
		// Hooks to process.
		foreach ( $this->hooks_to_process as $hook ) {
			add_action( $hook, array( $this, 'process_user_journey' ), 10 );
		}
	}

	/**
	 * Provider name/slug.
	 *
	 * @return string
	 * @since 1.0.2
	 *
	 */
	protected function get_provider() {
		return 'memberpress';
	}

	/**
	 * Process User Journey data once the order has been placed by the user.
	 *
	 * @param object $txn MemberPress Payment Transaction Object.
	 *
	 * @return void
	 * @since 1.0.2
	 *
	 */
	public function process_user_journey( $txn ) {

		$payment_id = $txn->id;

		$is_in_ga = $txn->get_meta( '_monsterinsights_is_in_ga', true );
		$skip_ga  = apply_filters( 'monsterinsights_memberpress_transaction_skip_user_journey', false, $txn );

		// If it's already in GA or filtered to skip, then skip adding
		if ( $skip_ga || 'yes' !== $is_in_ga ) {
			return;
		}

		$already_completed = $txn->get_meta( '_monsterinsights_user_journey_completed', true );

		if ( $already_completed && 'yes' === $already_completed ) {
			return;
		}

		$this->process_entry_meta( $payment_id, $txn );
	}
}

if ( MonsterInsights_User_Journey_Helper::is_memberpress_active() ) {
	new MonsterInsights_User_Journey_Process_MemberPress();
}
